function [p, f, SSH] = rslinhyp(y, X, d, A)
% RSLINHYP - Test the linear hypothesis H0: A*beta=d, where beta is the model parameters, against the alternative that A*beta is not equal to d. Version 0.1.15.
%
% The syntax is
% [p, f, SSH] = rslinhyp(y, X, d, A)
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of a predictors used in the regression, including a possible constant 1 column.
% A is an q by a matrix with linearly independent rows, forming the linear hypothesis to test.
% d is an q by a column vector forming the linear hypothesis to test.
%
% Return values
% p is the p-value of the null hypothesis.
% f is the corresponding F-test statistic.
% SSH is the hypothesis sum of squares.
%
% References
% Neter-Wasserman-Kutner: Applied Linear Regression Models, Richard D. Irwin Inc., 1983, Section 8.5.
% For background, see http://urn.fi/URN:NBN:fi:tty-201012131382 (Finnish, sorry!)
% some hints given in Section 2.4 of Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009.


% Copyright (C) 2015 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2015


% Check input
if nargin < 4 || nargin > 5,
    error('Try help rslinhyp for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rslinhyp for instructions.');
end
if cond(X'*X) > 1e6,
    warning('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end
if cond(A*A') > 1e10,
    warning('The rows of the matrix A seem to be linearly dependent. This gives the wrong number of degrees of freedom for the hypothesis sum of squares. Please trim your A.');
end
% Still check that SSE>0 so that testing is possible.
b=X\y;
pred=X*b;
r=y-pred;
SSE=r'*r;
if SSE < 1e-6,
    warning('Your model seems to fit the data perfectly, then no hypothesis testing is possible. If the equation X*b=y is in fact overdetermined, try normalizing the factor data.');
end

% Perform hypothesis test
[N,a]=size(X);
dfE=N-a;
MSE=SSE/dfE;
SSH=(A*b-d)'*(A*(X'*X)^-1*A')^-1*(A*b-d);
q=size(A,1);
f=SSH/q/MSE;
p=1-fcdf(f,q,dfE);

end